<?php
/**
 * A command for the login area (member/gamer area)
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class WebLoginAreaCommand extends BaseCommand implements Commandable {
	/**
	 * Name of the action
	 */
	private $actionName = "";

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this class
	 *
	 * @param	$resolverInstance	An instance of a command resolver class
	 * @return	$commandInstance	An instance a prepared command class
	 */
	public final static function createWebLoginAreaCommand (CommandResolver $resolverInstance) {
		// Get new instance
		$commandInstance = new WebLoginAreaCommand();

		// Set the application instance
		$commandInstance->setResolverInstance($resolverInstance);

		// Load general data like user status and such
		$commandInstance->prepareCommand();

		// Return the prepared instance
		return $commandInstance;
	}

	/**
	 * Prepares some general data which shall be displayed on every page
	 *
	 * @return	void
	 * @todo	Add some stuff here: Some personal data, app/game related data
	 */
	protected function prepareCommand () {
	}

	/**
	 * Executes the given command with given request and response objects
	 *
	 * @param	$requestInstance	An instance of a class with an Requestable interface
	 * @param	$responseInstance	An instance of a class with an Responseable interface
	 * @return	void
	 */
	public function execute (Requestable $requestInstance, Responseable $responseInstance) {
		// Get the action instance from registry
		$actionInstance = Registry::getRegistry()->getInstance('action');

		// Do we have an action here?
		if ($actionInstance instanceof PerformableAction) {
			// Execute the action (shall not output anything, see below why)
			$actionInstance->execute($requestInstance, $responseInstance);
		} // END - if

		// Get the application instance
		$appInstance = $this->getResolverInstance()->getApplicationInstance();

		// Prepare a template instance
		$templateInstance = $this->prepareTemplateInstance($appInstance);

		// Assign base URL
		$templateInstance->assignConfigVariable('base_url');

		// Assign all the application's data with template variables
		$templateInstance->assignApplicationData($appInstance);

		// Load the master template
		$masterTemplate = $appInstance->getMasterTemplate();

		// Load header template
		$templateInstance->loadCodeTemplate('header');

		// Compile and assign it with a variable
		$templateInstance->compileTemplate();
		$templateInstance->assignTemplateWithVariable('header', 'header');

		// Load footer template
		$templateInstance->loadCodeTemplate('footer');

		// Compile and assign it with a variable
		$templateInstance->compileTemplate();
		$templateInstance->assignTemplateWithVariable('footer', 'footer');

		// Load the matching template
		$templateInstance->loadCodeTemplate('action_' . $this->actionName);

		// Assign the template with the master template as a content ... ;)
		$templateInstance->compileTemplate();
		$templateInstance->assignTemplateWithVariable('action_' . $this->actionName, 'login_content');

		// Load main template
		$templateInstance->loadCodeTemplate('login_main');

		// Assign the main template with the master template as a content ... ;)
		$templateInstance->compileTemplate();
		$templateInstance->assignTemplateWithVariable('login_main', 'content');

		// Load the master template
		$templateInstance->loadCodeTemplate($masterTemplate);

		// Set title
		$templateInstance->assignVariable('title', $this->getLanguageInstance()->getMessage($this->actionName . '_title'));

		// ... and all variables. This should be merged together in a pattern
		// to make things easier. A cache mechanism should be added between
		// these two calls to cache compiled templates.
		$templateInstance->compileVariables();

		// Get the content back from the template engine and put it in response class
		$templateInstance->transferToResponse($responseInstance);
	}

	/**
	 * Adds extra filters to the given controller instance. An corresponding action class must now exist!
	 *
	 * @param	$controllerInstance		A controller instance
	 * @param	$requestInstance		An instance of a class with an Requestable interface
	 * @return	void
	 */
	public function addExtraFilters (Controller $controllerInstance, Requestable $requestInstance) {
		// Default is no action
		$actionInstance = null;

		// Get registry
		$registryInstance = Registry::getRegistry();

		// Get our application instance from the registry
		$appInstance = $registryInstance->getInstance('application');

		// Default action is the one from configuration
		$this->actionName = sprintf("%s_login_%s",
			str_replace("-", "_", $appInstance->getAppShortName()),
			$this->getConfigInstance()->readConfig('login_default_action')
		);

		// Get "action" from request
		$actReq = $requestInstance->getRequestElement('action');

		// Do we have a "action" parameter set?
		if ((is_string($actReq)) && (!empty($actReq))) {
			// Then use it with prefix
			$this->actionName = sprintf("%s_login_%s",
				str_replace("-", "_", $appInstance->getAppShortName()),
				$actReq
			);
		} // END - if

		// Get application instance
		$applicationInstance = $this->getResolverInstance()->getApplicationInstance();

		// Get a resolver
		$actionResolver = WebActionResolver::createWebActionResolver($this->actionName, $applicationInstance);

		// Resolve the action
		$actionInstance = $actionResolver->resolveAction();

		// Add more action-specific filters
		$actionInstance->addExtraFilters($controllerInstance, $requestInstance);

		// Remember this action in registry
		Registry::getRegistry()->addInstance('action', $actionInstance);
	}
}

// [EOF]
?>
